/*
 * A pipe paging coordinator, permitting memory sharing pipe endpoints.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <set>

#include <fsserver/notification.h>
#include <fsserver/page_mapper.h>
#include <fsserver/pages.h>
#include <fsserver/pipe_accessor.h>



/* Pipe paging support, maintaining the sequence of active regions or sections
   in a pipe. */

class PipePaging : public NotificationSupport
{
protected:
    Memory *_memory;
    Pages *_pages;
    PageQueue *_queue;

    /* Regions acting as files with their own accessors. */

    PageMapper *_regions[2];
    PipeAccessor _accessors[2];

    /* The first region is initially exposed to both reader and writer. */

    int _reading = 0, _writing = 0;

    /* Pipe section/region size. */

    offset_t _size;

    /* Endpoint status. */

    unsigned int _active_endpoints = 2;

    /* Common functionality. */

    virtual PageMapper *_add_region();

    virtual PageMapper *_next_region();

    virtual void discard_region(unsigned int i);

public:
    explicit PipePaging(Memory *memory, offset_t size);

    virtual ~PipePaging();

    virtual unsigned int detach();

    virtual offset_t region_size()
    { return _size; }

    /* Region management. */

    virtual PageMapper *add_region();

    virtual PageMapper *current_region();

    virtual PageMapper *current_region(bool writing);

    virtual PageMapper *next_region();

    /* Access management. */

    virtual int closed();

    /* Special values for various arrays. */

    enum PipePagingRoles : unsigned int
    {
        READER = 0,
        WRITER = 1,
    };
};

// vim: tabstop=4 expandtab shiftwidth=4
